package com.democompany.services.esb.customj;

import java.io.StringReader;
import java.io.StringWriter;

import com.democompany.services.esb.customj.util.AbstractXQService;
import com.democompany.services.esb.util.exception.ValidationException;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;
import com.unidex.xflat.XflatException;
import com.unidex.xflat.XmlConvert;

/**
 * This Custom Java Service uses XMLConvert library to convert XML to flat file
 * formats and flat files to XML. The XMLConvert library uses special XML
 * configuration files to describe the flat file gramar. The syntax is flexible
 * and allows expressing large number of formats including CSV, fixed length,
 * Windows INI, custom record layouts, etc. The service accepts two inputs: an
 * XML document describing the schema of a flat file and an input file to
 * convert. If the input file is an XML document, it will be converted to a flat
 * file. If an input is a flat file, it will be converted to XML.
 * 
 * @author Leon Rubin
 * 
 */
public class XMLConvert extends AbstractXQService {
	private static final String TEXT_PART = "text";
	private static final String XML_PART = "xml";
	private static final String OUTPUT_PART = "output";
	private static final String FLAT_SCHEMA_PART = "flatSchema";

	/**
	 * Constructor for a XMLConvert.
	 */
	public XMLConvert() {
		super("XMLConvert");
	}

	/**
	 * This method extracts the input from each request and performs the
	 * conversion form XML to flat file or visa versa (depending on inputs). The
	 * method looks for part called "flatSchema" which contains the flat file
	 * format definition. Then, it looks for parts called "text" or "xml". The
	 * ouptput is placed in a new part called "output".
	 */
	protected void processMessage(XQServiceContext ctx, XQEnvelope env)
			throws XQServiceException, XQMessageException {
		try {
			XQMessage msg = env.getMessage();

			if (msg.doesPartExist(FLAT_SCHEMA_PART) == false) {
				throw new ValidationException(
						"Request must contain a flat file schema");
			}

			if (msg.doesPartExist(TEXT_PART) == false
					&& msg.doesPartExist(XML_PART) == false) {
				throw new ValidationException(
						"No input detected.  A request must contain a part with plain text or XML input");
			}

			if (msg.doesPartExist(TEXT_PART) && msg.doesPartExist(XML_PART)) {
				throw new ValidationException(
						"A request must contain only one input part.  This request appears to have both text and XML input");
			}

			StringWriter outputWriter = new StringWriter();
			StringReader inputReader = null;
			String flatSchema = (String) env.getMessage().getPart(
					FLAT_SCHEMA_PART).getContent().toString();
			XmlConvert convert = new XmlConvert(new StringReader(flatSchema),
					true);

			if (msg.doesPartExist(TEXT_PART)
					&& msg.doesPartExist(XML_PART) == false) {
				inputReader = new StringReader((String) msg.getPart(TEXT_PART)
						.getContent());

				convert.flatToXml(inputReader, outputWriter);

			} else if (msg.doesPartExist(TEXT_PART) == false
					&& msg.doesPartExist(XML_PART)) {
				inputReader = new StringReader((String) msg.getPart(XML_PART)
						.getContent());

				convert.xmlToFlat(inputReader, outputWriter);
			}

			XQPart outputPart = msg.createPart();

			outputPart.setContentId(OUTPUT_PART);
			outputPart.setContent(outputWriter.toString(), "text");
			msg.addPart(outputPart);
		} catch (ValidationException e) {
			throw new XQServiceException("Request failed validation", e);
		} catch (XflatException e) {
			throw new XQServiceException(
					"Failed to convert from XML to flat file format (or visa versa)",
					e);
		}
	}
}